/*
  A6Message.cxx

  last updated : 2017.02.09
*/
//#include
#include <cstring>
#include <cstdio>
#include <stdlib.h>
#include <iostream>
#include "CommBase/A6Message.hxx"

A6Message::A6Message(int type) {
  std::memset(mData, 0, sizeof(mData));
  setType(type);
}

A6Message::A6Message() {
  std::memset(mData, 0, sizeof(mData));
}

A6Message::~A6Message() {
}


void A6Message::setPayload(const char* payload_c, int plsize) {
  std::memcpy(&mData[headerSize()], payload_c, plsize);
  setPayloadSize(plsize);
  return;
}

void A6Message::appendPayload(const char* s, int nbytes) {
  int plsize = payloadSize();
  std::memcpy(&mData[headerSize()+plsize], s, nbytes);
  setPayloadSize(plsize + nbytes);
  return;
}

int A6Message::type() const {
  int type_i = 0x0;
  type_i = mData[0] & 0xff;
  return type_i;
}

int A6Message::commandID() const {
  int cmdid_i = 0x0;
  cmdid_i = mData[1] & 0xff;
  return cmdid_i;
}

int A6Message::payloadSize() const {
  int plsize_i = 0x0;
  plsize_i = mData[2] & 0xff;
  return plsize_i;
}

const char* A6Message::payload() const {
  return &mData[headerSize()];
}

int A6Message::packetSize() const {
  int size=0;
  size = headerSize() + payloadSize();
  return size;
}

void A6Message::dump(std::ostream& out) {
  out << "--- A6Message Contents ---" << std::endl;
  out << "  Message Type  : " << type() << std::endl;
  out << "  Command ID    : " << commandID() << std::endl;
  out << "  Payload size  : " << payloadSize() << std::endl;
  out << "  packet size   : " << packetSize() << std::endl;

  char line[100];
  unsigned char* ucdata = reinterpret_cast<unsigned char*>(mData);
  int nwords = packetSize() / 4;
  if ( (packetSize() % 4) != 0) {
    nwords++;
  }
  int iword = 0;
  for (unsigned int i=0; i<=static_cast<unsigned int>(packetSize()); i+=4) {
    std::sprintf(line, "[%04d] 0x%02x%02x%02x%02x",
		 iword++, ucdata[i], ucdata[i+1], ucdata[i+2], ucdata[i+3]);
    // out << line;
    // out << "  ";
    // std::sprintf(line, "%c%c%c%c",
    // 		 ucdata[i], ucdata[i+1], ucdata[i+2], ucdata[i+3]);
    out << line << std::endl;
  }
  out << std::endl;
  return;
}

void A6Message::clear() {
  std::memset(mData, 0, sizeof(mData));
  return;
}

void A6Message::clearPayload() {
  std::memset(&mData[headerSize()], 0, packetSize()-headerSize());
  setPayloadSize(0x0);
  return;
}

