/*
  CommandSet.cxx
*/
#include "CommBase/CommandSet.hxx"
#include <cstring>
#include <cstdio>
#include <iostream>
#include <stdlib.h>

CommandSet::CommandSet(int type) {
  std::memset(mData, 0, sizeof(mData));
  setType(type);
}

CommandSet::CommandSet() {
  std::memset(mData, 0, sizeof(mData));
}

CommandSet::~CommandSet() {
}

void CommandSet::setAddress(unsigned int i) {
  mData[4] = ((i>>24) & 0xff);
  mData[5] = ((i>>16) & 0xff);
  mData[6] = ((i>>8) & 0xff);
  mData[7] = (i & 0xff);
  return;
}

void CommandSet::setPayload(const char* payload_c, int plsize) {
  std::memcpy(&mData[headerSize()], payload_c, plsize);
  setPayloadSize(plsize);
  return;
}

void CommandSet::appendPayload(const char* s, int nbytes) {
  int plsize = payloadSize();
  std::memcpy(&mData[headerSize()+plsize], s, nbytes);
  setPayloadSize(plsize + nbytes);
  return;
}

int CommandSet::type() const {
  int type_i = 0x0;
  type_i = (mData[0] & 0xff);
  return type_i;
}

int CommandSet::rw() const {
  int rw_i = 0x0;
  rw_i = (mData[1] & 0xff);
  return rw_i;
}

int CommandSet::commandID() const {
  int cmdid_i = 0x0;
  cmdid_i = mData[2] & 0xff;
  return cmdid_i;
}

int CommandSet::payloadSize() const {
  int plsize_i = 0x0;
  plsize_i = mData[3] & 0xff;
  return plsize_i;
}

unsigned int CommandSet::address() const {
  unsigned int address_i = 0x0;
  unsigned int tmp=0;
  for (int i=0; i<4; ++i) {
    tmp = (mData[4+i] & 0xff);
    address_i |= (tmp<<(24-8*i));
  }
  return address_i;
}

const char* CommandSet::payload() const {
  return &mData[headerSize()];
}

int CommandSet::packetSize() const {
  int size=0;
  size = headerSize() + payloadSize();
  return size;
}

void CommandSet::dump(std::ostream& out) {
  out << "--- CommandSet Contents ---" << std::endl;
  out << "  Message Type  : " << type() << std::endl;
  out << "  Read or Write : " << rw() << std::endl;
  out << "  Command ID    : " << commandID() << std::endl;
  out << "  Payload size  : " << payloadSize() << std::endl;
  out << "  Address       : " << address() << std::endl;
  out << "  packet size   : " << packetSize() << std::endl;

  char line[100];
  unsigned char* ucdata = reinterpret_cast<unsigned char*>(mData);
  int nwords = packetSize() / 4;
  if ( (packetSize() % 4) != 0) {
    nwords++;
  }
  int iword = 0;
  for (unsigned int i=0; i<=static_cast<unsigned int>(packetSize()); i+=4) {
    std::sprintf(line, "[%04d] 0x%02x%02x%02x%02x",
		 iword++, ucdata[i], ucdata[i+1], ucdata[i+2], ucdata[i+3]);
    // out << line;
    // out << "  ";
    // std::sprintf(line, "%c%c%c%c",
    // 		 ucdata[i], ucdata[i+1], ucdata[i+2], ucdata[i+3]);
    out << line << std::endl;
  }
  out << std::endl;
  return;
}

void CommandSet::clear() {
  std::memset(mData, 0, sizeof(mData));
  return;
}

void CommandSet::clearPayload() {
  std::memset(&mData[headerSize()], 0, packetSize()-headerSize());
  setPayloadSize(0x0);
  return;
}


