/*
  DaqControllerH.cxx

  last updated : 2017.02.13
*/
#include "PixelDAQ/DaqControllerH.hxx"
#include "CommBase/GlobalConfig.hxx"
#include "CommBase/CommandSet.hxx"

DaqControllerH::DaqControllerH() {
}

DaqControllerH::~DaqControllerH() {
}

int DaqControllerH::initialize() {
  mCM_tcp.createBuffer(10000, "TCP"); // fake
  mCM_udp.createBuffer(10000, "UDP");
  return 0;
}

int DaqControllerH::configure() {
  // IPAddress must be set before initialize
  int ret=0;
  ret = mCM_tcp.connect();
  ret = mCM_udp.createUDPSocket();
  return ret;
}

int DaqControllerH::finalize() {
  return 0;
}

/* ------------------------------------------------------- */
/*              Command for FE-I4 chips (UDP)              */
/* ------------------------------------------------------- */
int DaqControllerH::sendUDPCommand(int rw, int cmdid, int plsize,
			       int address, char* payload) {
  GlobalConfig* gconfig = GlobalConfig::get();
  CommandSet cmd(0xff);
  cmd.setRW(rw);
  cmd.setCommandID(cmdid);
  cmd.setAddress(address);
  cmd.setPayload(payload, plsize);

  if (gconfig && gconfig->output(GlobalConfig::kDEBUG3)) {
    std::cout<<"---send a command to the server...."<<std::endl;
  }
  if (gconfig && gconfig->output(GlobalConfig::kVERBOSE)) {
    cmd.dump(std::cout);
  }
  mCM_udp.write(cmd);
  return 0;
}

int DaqControllerH::writeUDPCommand(int cmdid, int plsize, int address,
				char* payload) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG3)) {
    std::cout<<"CMD >> Write Register (add:"<< address << ")"<<std::endl;
  }
  sendUDPCommand(kWRITE, cmdid, plsize, address, payload);

  /* read acknowledgement */
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG3)) {
    std::cout<<"---reading a message from the server...."<<std::endl;
  }
  CommandSet cmd;
  mCM_udp.read(cmd);
  if (gconfig && gconfig->output(GlobalConfig::kVERBOSE)) {
    cmd.dump(std::cout);
  }
  if (cmd.rw()==kACK) {
    if (gconfig && gconfig->output(GlobalConfig::kVERBOSE)) {
      std::cout<<"received acknowledgement"<<std::endl;
    }
  } else {
    return -1;
  }
  return 0; 
}

unsigned int DaqControllerH::readUDPCommand(int cmdid, int address) {
   GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG3)) {
    std::cout<<"CMD >> Read Register (add:"<< address << ")"<<std::endl;
  }
  char payload[1] = { 0x0 };
  sendUDPCommand(kREAD, cmdid, 0x1/* not 0 */, address, payload);

  /* read address data */
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG3)) {
    std::cout<<"---reading a message from the server...."<<std::endl;
  }
  CommandSet cmd;
  mCM_udp.read(cmd);
  if (gconfig && gconfig->output(GlobalConfig::kVERBOSE)) {
    cmd.dump(std::cout);
  }
  if (cmd.rw()==kREPLY) {
    if (gconfig && gconfig->output(GlobalConfig::kVERBOSE)) {
      std::cout<<"received register data"<<std::endl;
    }
    const char* data = cmd.payload();
    unsigned int regval = data[0] & 0xff;
    if (gconfig && gconfig->output(GlobalConfig::kVERBOSE)) {
      std::cout<<"address:"<< cmd.address() <<" = "<< regval <<std::endl;
    }
  } else {
    return 1;
  }
  return 0; 
}

/* ------------------------------------------------------- */
/*                Command for FE-I4 chips                  */
/* ------------------------------------------------------- */

int DaqControllerH::BCReset() {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout<<"--CMD >> Bunch Counter Reset"<<std::endl;
  }
  char payload[2]; 
  payload[0]=(mMask & 0xff);
  payload[1]=0x01;
  writeUDPCommand(kBCReset, 0x2, kBCReset, payload);
  return 0;
} 

int DaqControllerH::ECReset() {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout<<"--CMD >> Event Counter Reset"<<std::endl;
  }
  char payload[2]; 
  payload[0]=(mMask & 0xff);
  payload[1]=0x01;
  writeUDPCommand(kECReset, 0x2, kECReset, payload);

  return 0;
}

unsigned int DaqControllerH::readRegister(int chipID, int gr_address) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout << "--CMD >> Read Register...." << std::endl;
  }
  char payload[3];
  payload[0]=(chipID & 0xff);
  payload[1]=(gr_address & 0xff);
  payload[2]=0x1;
  writeUDPCommand(kRdRegister, 0x3, kRdRegister, payload);
 
 if (gconfig && gconfig->output(GlobalConfig::kDEBUG3)) {
    std::cout << "---Received a message from the server ..." << std::endl;
  }

  CommandSet cmd;
  mCM_tcp.read(cmd);
  cmd.dump(std::cout);

  /*--------- decode---------*/
  // to be updated...

  return 0;
}

int DaqControllerH::writeRegister(int chipID, int gr_address,
				 unsigned short data) {
  //sleep(1);
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout<<"--CMD >> Write Register"<<std::endl;
  }
  char payload[5];
  payload[0]=(chipID & 0xff);
  payload[1]=(gr_address & 0xff);
  payload[2]=((data>>8) & 0xff);
  payload[3]=(data & 0xff);
  payload[4]=0x01;
  writeUDPCommand(kWrRegister, 0x5, kWrRegister, payload);
  return 0;
}

int DaqControllerH::globalReset(int chipID) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout<<"--CMD >> Global Reset"<<std::endl;
  }
  char payload[2];
  payload[0]=(chipID & 0xff);
  payload[1]=0x01;
  writeUDPCommand(kGlobalReset, 0x2, kGlobalReset, payload);
  return 0;
}

int DaqControllerH::globalPulse(int chipID, int width) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout<<"--CMD >> Global Pulse"<<std::endl;
  }
  char payload[3];
  payload[0]=(chipID & 0xff);
  payload[1]=(width & 0xff);
  payload[2]=0x01;
  writeUDPCommand(kGlobalPulse, 0x3, kGlobalPulse, payload);
  return 0;
}

int DaqControllerH::setRunMode(int chipID) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout<<"--CMD >> Set RunMode"<<std::endl;
  }
  char payload[3];
  payload[0]=(chipID & 0xff);
  payload[1]=0x38;
  payload[2]=0x01;
  writeUDPCommand(kSetRunMode, 0x3, kSetRunMode, payload);
  return 0;
}

int DaqControllerH::setConfMode(int chipID) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout<<"--CMD >> Set ConfMode"<<std::endl;
  }
  char payload[3];
  payload[0]=(chipID & 0xff);
  payload[1]=0x07;
  payload[2]=0x01;
  writeUDPCommand(kSetConfMode, 0x3, kSetConfMode, payload);
  return 0;
}

/* ------------------------------------------------------- */
/*                  Command for firmware                   */
/* ------------------------------------------------------- */
unsigned int DaqControllerH::readStatus(int add) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout<<"--CMD >> Read Register Status in Firmware"<<std::endl;
  }
  unsigned int data = readUDPCommand(add/*command ID*/, add);
  return data;
}

int DaqControllerH::sendChipMask(int chipMask) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout<<"--CMD >> Send Chip Mask"<<std::endl;
  }
  if (chipMask>15) {
    if (gconfig && gconfig->output(GlobalConfig::kERROR)) {
      std::cout<<"Error!! (sendChipMask()) "
	       <<"chipMask should be in 0000-1111 (0-15)."<<std::endl;
    }
    return -1;
  }
  char payload[1] = { chipMask & 0xff };
  writeUDPCommand(0x21, 0x01, 0x21, payload);
  return 0;
}

int DaqControllerH::sendChipID(int chip_index, int chipID) {
  if (chip_index>3) {
    std::cout<<"Error!! (sendChipID()) chipIndex should be in 0-3."<<std::endl;
    return -1;
  }
  if (chip_index>15) {
    std::cout<<"Error!! (sendChipID()) "
	     <<"chipID should be in 0000-1111 (0-15)."<<std::endl;
    return -1;
  }
  char payload[1] = { ((chipID<<4)+chip_index) & 0xff };
  writeUDPCommand(0x22+chip_index, 0x1, 0x22+chip_index, payload);
  return 0;
}

int DaqControllerH::chkCmdStatus(int address) {
  //repeat checking the command status in FPGA
  //If it's zero, the command execution in FPGA is finished.
  int nrpt = 0;
  while(1){
    unsigned int val = readStatus(address);
    // std::cout<<"readStatus() : val = "<< val <<std::endl;
    if(val==0) break;
  }
  return nrpt;
}

int DaqControllerH::chkDoutReadiness(int chip) {
  if (chip>3) {
    std::cout<<"Error!! (chkDoutReadiness()) chip should be in 0-3."<<std::endl;
    return -1; 
  }
  int irpt=0;
  int nrpt=50000;
  for (irpt=0; irpt<nrpt; ++irpt) {
    unsigned int checkedBit = 0x1<<chip;
    unsigned int val = readStatus(0x20);
    // std::cout<< checkedBit <<", "<< val <<std::endl;
    if(val&checkedBit) break;
    usleep(50);
  }  
  return (irpt==nrpt) ? -1 : 1;
}


/* ------------------------------------------------------- */
/*                     Special Command                     */
/* ------------------------------------------------------- */
int DaqControllerH::enableDout(int chipID) {
  //writeRegister(chipID, 20, 0x2641);
  writeRegister(chipID,  1, 0x0000);
  writeRegister(chipID,  2, 0x2800);
  writeRegister(chipID,  3, 0xffff);
  writeRegister(chipID,  4, 0xffff);
  writeRegister(chipID,  5, 0xd4ff);
  writeRegister(chipID,  6, 0x7cd4);
  writeRegister(chipID,  7, 0xff58);
  writeRegister(chipID,  8, 0xf258);
  writeRegister(chipID,  9, 0x00aa);
  writeRegister(chipID, 10, 0x4c4c);
  writeRegister(chipID, 11, 0x56d4);
  writeRegister(chipID, 12, 0x7800);
  //  writeRegister(chipID, 13, 0x);
  writeRegister(chipID, 14, 0xd5b7);
  writeRegister(chipID, 15, 0x1a96);
  writeRegister(chipID, 16, 0x0038);
  writeRegister(chipID, 17, 0x002d);
  writeRegister(chipID, 18, 0x00ff);
  writeRegister(chipID, 19, 0x0600);
  writeRegister(chipID, 20, 0x0041);
  //  writeRegister(chipID, 21, 0x);
  //  writeRegister(chipID, 22, 0x);
  writeRegister(chipID, 23, 0x0000);
  writeRegister(chipID, 24, 0x0000);
  writeRegister(chipID, 25, 0x0500);
  writeRegister(chipID, 26, 0x0048);
  writeRegister(chipID, 27, 0x8000);
  writeRegister(chipID, 28, 0x8206);
  writeRegister(chipID, 29, 0x2007);
  writeRegister(chipID, 30, 0x0000);
  writeRegister(chipID, 31, 0xf400);
  writeRegister(chipID, 32, 0x0000);
  writeRegister(chipID, 33, 0x0000);
  writeRegister(chipID, 34, 0x000d);
  //  writeRegister(chipID, 35, 0x);
  return 0;
}
