/*
  DaqControllerN.cxx

  last updated : 2017.02.14
*/
#include "PixelDAQ/DaqControllerN.hxx"
#include "CommBase/GlobalConfig.hxx"
#include "CommBase/BitsetHelper.hxx"
#include "CommBase/A6Message.hxx"
#include "PixelDAQ/Decoder.hxx"

DaqControllerN::DaqControllerN() {
}

DaqControllerN::~DaqControllerN() {
}

int DaqControllerN::initialize() {
  std::cout<<"DaqControllerN::initialize()"<<std::endl;
  mCM_tcp.createBuffer(10000, "TCP");
  // mCM_udp.createBuffer(10000, "UDP");
  mStatus.set(0);
  return 0;
}

int DaqControllerN::configure() {
  std::cout<<"DaqControllerN::configure()"<<std::endl;
  // IPAddress must be set before initialize
  if (!mStatus.test(0)) {
    std::cout<<"DaqController::configure() : ";
    std::cout<<"Retry AFTER calling DaqController::initialize()"<<std::endl;
    return -1;
  }
  int ret=0;
  ret = mCM_tcp.connect();
  // ret = mCM_udp.createUDPSocket();
  mStatus.set(1);
  return ret;
}

int DaqControllerN::finalize() {
  mStatus.reset();
  return 0;
}

int DaqControllerN::sendCommand(int cmdid, int plsize, char* payload) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (mStatus.count()<2) {
    if (gconfig && gconfig->output(GlobalConfig::kERROR)) {
      std::cerr<<"ERROR in DaqController::sendCommand(...) : ";
      std::cerr<<"Status error"<<std::endl;
    }
    return -1;
  }
  A6Message msg(0xa6);
  msg.setCommandID(cmdid);
  msg.setPayload(payload, plsize);

  if (gconfig && gconfig->output(GlobalConfig::kDEBUG3)) {
    std::cout<<"---send a command to the server...."<<std::endl;
  }
  if (gconfig && gconfig->output(GlobalConfig::kVERBOSE)) {
    msg.dump(std::cout);
  }
  mCM_tcp.write(msg);
  return 0;
}

void bitset2chararray(char* dest, const std::bitset<39>& src) {		      
  unsigned int n=39, n2=40;
  unsigned int ibyte, ibit, j, bitvalue=0;
  char c=0;
  for (unsigned int i=0; i<n2; ++i) {
    j= n-1-i;
    ibyte = i/8;
    ibit = (7- i%8);
    if ( (i%8)==0 ) {
      c = 0;
    }
    bitvalue=0;
    if (i<n) {
      if (src.test(j))  bitvalue = 1;
    }
    c |= (bitvalue << ibit);
    if ( (i%8)==7 ) {
      dest[ibyte] = c;
    }
  }
}

/* ------------------------------------------------------- */
/*              Command for FE-I4 chips (TCP)              */
/* ------------------------------------------------------- */

int DaqControllerN::sendFastCommand(int type) {
  std::bitset<9> bs_field1(0x16);
  std::bitset<9> bs_field2(static_cast<unsigned int>(type));
  std::bitset<9> bs_payload = (bs_field1<<4) | (bs_field2);
  char payload[2];
  payload[0] = subbits(bs_payload, 8, 1).to_ulong() & 0xff;
  payload[1] = bs_payload.test(0) ? 0x80 : 0x0;
  sendCommand(kChipCommand, 2, payload);
  return 0;
}

int DaqControllerN::sendSlowCommand(int type, int chipid, int field5, 
				   unsigned short field6) {
  std::bitset<39> bs_field1(0x16);
  std::bitset<39> bs_field2(0x08);
  std::bitset<39> bs_field3(static_cast<unsigned int>(type));
  std::bitset<39> bs_field4(static_cast<unsigned int>(chipid));
  std::bitset<39> bs_field5(static_cast<unsigned int>(field5));
  std::bitset<39> bs_field6(field6);
  std::bitset<39> bs_payload = (bs_field1<<34) | (bs_field2<<30)
    | (bs_field3<<26) | (bs_field4<<22) | (bs_field5<<16) | (bs_field6);
  char payload[5];
  bitset2chararray(payload, bs_payload);
  sendCommand(kChipCommand, 5, payload);
  return 0;
}

int DaqControllerN::BCReset() {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout << "--CMD >> Bunch Counter Reset" << std::endl;
  }
  sendFastCommand(0x1); 
  return 0;
}

int DaqControllerN::ECReset() {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout << "--CMD >> Event Counter Reset" << std::endl;
  }
  sendFastCommand(0x2); 
  return 0;
}

int DaqControllerN::calPulse() {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout << "--CMD >> Calibration Pulse" << std::endl;
  }
  sendFastCommand(0x4); 
  return 0;
}

unsigned int DaqControllerN::readRegister(int chipID, int gr_address) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout << "--CMD >> Read Register...." << std::endl;
  }
  sendSlowCommand(0x1, chipID, gr_address, 0x0);

  A6Message msg;
  Decoder decoder;
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG3)) {
    std::cout << "---Received a message from the server ..." << std::endl;
  }
  /* ----- to be updated ----- */
  mCM_tcp.read(msg);
  decoder.decode(msg.data());
  short address = decoder.address();

  mCM_tcp.read(msg);
  decoder.decode(msg.data());
  short value = decoder.value();
  /* ------------------------- */
  return static_cast<unsigned int>(value);
}

int DaqControllerN::writeRegister(int chipID, int gr_address,
				 unsigned short data) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout<<"--CMD >> Write Register"<<std::endl;
  }
  sendSlowCommand(0x2, chipID, gr_address, data);
  return 0;
}

int DaqControllerN::globalReset(int chipID) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout<<"--CMD >> Global Reset"<<std::endl;
  }
  sendSlowCommand(0x8, chipID, 0x0, 0x0);
  return 0;
}

int DaqControllerN::globalPulse(int chipID, int width) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout<<"--CMD >> Global Pulse"<<std::endl;
  }
  sendSlowCommand(0x9, chipID, width, 0x0);
  return 0;
}

int DaqControllerN::setRunMode(int chipID) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout<<"--CMD >> Set RunMode"<<std::endl;
  }
  sendSlowCommand(0xa, chipID, 0x38, 0x0);
  return 0;
}

int DaqControllerN::setConfMode(int chipID) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout<<"--CMD >> Set ConfMode"<<std::endl;
  }
  sendSlowCommand(0xa, chipID, 0x7, 0x0);
  return 0;
}

/* ------------------------------------------------------- */
/*                  Command for firmware                   */
/* ------------------------------------------------------- */
unsigned int DaqControllerN::readStatus(int add) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kDEBUG2)) {
    std::cout<<"--CMD >> Read Register Status in Firmware"<<std::endl;
  }
  char payload[1] = { add & 0xff };
  sendCommand(kInRegCommand, 0x1, payload);
  
  A6Message msg;
  Decoder decoder;
  mCM_tcp.read(msg);
  char* c_data = msg.data();
  int address = c_data[0] & 0xff;
  unsigned int value = c_data[1] & 0xff;
  std::cout<<"address: "<< address <<", value: "<< value <<std::endl; 
  return value;
}

/* ------------------------------------------------------- */
/*                    Special Command                      */
/* ------------------------------------------------------- */

int DaqControllerN::enableDout(int chipID) {
  writeRegister(chipID, 20, 0x2641);
  writeRegister(chipID,  1, 0x0000);
  return 0;
  writeRegister(chipID,  2, 0x2800);
  writeRegister(chipID,  3, 0xffff);
  writeRegister(chipID,  4, 0xffff);
  writeRegister(chipID,  5, 0xd4ff);
  writeRegister(chipID,  6, 0x7cd4);
  writeRegister(chipID,  7, 0xff58);
  writeRegister(chipID,  8, 0xf258);
  writeRegister(chipID,  9, 0x00aa);
  writeRegister(chipID, 10, 0x4c4c);
  writeRegister(chipID, 11, 0x56d4);
  writeRegister(chipID, 12, 0x7800);
  //  writeRegister(chipID, 13, 0x);
  writeRegister(chipID, 14, 0xd5b7);
  writeRegister(chipID, 15, 0x1a96);
  writeRegister(chipID, 16, 0x0038);
  writeRegister(chipID, 17, 0x002d);
  writeRegister(chipID, 18, 0x00ff);
  writeRegister(chipID, 19, 0x0600);
  writeRegister(chipID, 20, 0x0041);
  //  writeRegister(chipID, 21, 0x);
  //  writeRegister(chipID, 22, 0x);
  writeRegister(chipID, 23, 0x0000);
  writeRegister(chipID, 24, 0x0000);
  writeRegister(chipID, 25, 0x0500);
  writeRegister(chipID, 26, 0x0048);
  writeRegister(chipID, 27, 0x8000);
  writeRegister(chipID, 28, 0x8206);
  writeRegister(chipID, 29, 0x0007);
  writeRegister(chipID, 30, 0x0000);
  writeRegister(chipID, 31, 0xf400);
  writeRegister(chipID, 32, 0x0000);
  writeRegister(chipID, 33, 0x0000);
  writeRegister(chipID, 34, 0x000d);
  //  writeRegister(chipID, 35, 0x);
  return 0;
}
