/*
  Decoder.cxx   for FE-I4B

  last updated : 2017.02.02
*/
#include "CommBase/GlobalConfig.hxx"
#include "CommBase/BitsetHelper.hxx"
#include "PixelDAQ/Decoder.hxx"

Decoder::Decoder() {
  clear();
}

Decoder::~Decoder() {
}

int Decoder::decode(const char* c_data) {
  GlobalConfig* gconfig = GlobalConfig::get();

  std::bitset<24> bs_data(0);
  for (int ic=0; ic<3; ++ic) {
    std::bitset<24> tmp(c_data[ic] & 0xff);
    bs_data |= tmp<<(16-8*ic);
  }
  if (gconfig && gconfig->output(GlobalConfig::kVERBOSE)) {
    std::cout<<"bs_data = "<< bs_data << std::endl;
  }
  mField1 = subbits(bs_data, 23, 19).to_ulong();
  if (mField1==0x1d) /* not Data Record */ {
    mField2 = subbits(bs_data, 18, 16).to_ulong();
    switch (mField2) {
    case 0x1: // Data Header
      mTrigID = subbits(bs_data, 11, 8).to_ulong();
      mBCID = subbits(bs_data, 7, 0).to_ulong();
      break;
    case 0x2: // Address Recode
      mRegType = bs_data[15];
      mAddress = subbits(bs_data, 14, 0).to_ulong();
      break;
    case 0x4: // Value Recode
      mRegVal = subbits(bs_data, 15, 0).to_ulong();
      break;
    case 0x7: // Service Recode
      mSR = subbits(bs_data, 15, 10).to_ulong();
      mSR_pl = subbits(bs_data, 9, 0).to_ulong();
      break;
    default:
      if (gconfig && gconfig->output(GlobalConfig::kERROR)) {
	std::cout<<"<<< Decode Error! >>> field2 = "<< mField2
		 <<" : This Recode Word is not defined."<<std::endl;
      }
      break;
    }
  } else /* Data Recode */ {
    mColumn = subbits(bs_data, 23, 17).to_ulong();
    mRow = subbits(bs_data, 16, 8).to_ulong();
    mToT_top = subbits(bs_data, 7, 4).to_ulong();
    mToT_bot = subbits(bs_data, 3, 0).to_ulong();
  }
  
  if (gconfig && gconfig->output(GlobalConfig::kVERBOSE)) {
    info(std::cout);
  }
  
  return 0;
}

void Decoder::clear() {
  mChipID=0; mField1=0; mField2=0; mTrigID=0; mBCID=0;
  mColumn=0; mRow=0; mToT_top=0; mToT_bot=0;
  mRegType=0; mAddress=0; mRegVal=0;
  mSR=0; mSR_pl=0;
  return;
}

void Decoder::info(std::ostream& out) {
  out << "----- Decoder Information -----"<<std::endl;
  out << "Chip ID    : "<< mChipID <<std::endl;

  if (mField1==0x1d) {
    switch (mField2) {
    case 0x1:  
      out << "Trigger ID : "<< mTrigID <<std::endl;
      out << "BC ID      : "<< mBCID <<std::endl;
      break;
    case 0x2:
      out << "Reg Type   : "<< mRegType <<std::endl;
      out << "Address    : "<< mAddress <<std::endl;
      break;
    case 0x4:
      out << "Value      : "<< mRegVal <<std::endl;
      break;
    case 0x7:
      out << "Service Recode : "<< mSR <<std::endl;
      if (mSR>=14 && mSR<=16) {
	out << "Message        : "<< mSR_pl <<std::endl;
      } else {
	out << "Error Counter  : "<< mSR_pl <<std::endl;
      }
      break;
    default:
      break;
    }
  } else {
    out << "Column     : "<< mColumn <<std::endl;
    out << "Row        : "<< mRow <<std::endl;
    out << "ToT top    : "<< mToT_top <<std::endl;
    out << "ToT bot    : "<< mToT_bot <<std::endl;
  }
  out <<std::endl;
  return;
}


