/*
  Operation.cxx

  last updated : 2017.02.14
*/
#include <iostream>
#include <sstream>
#include <fstream>
#include <cstdio>
#include <stdlib.h>
#include "CommBase/GlobalConfig.hxx"
#include "PixelDAQ/Operation.hxx"

Operation::Operation() : mDaq(0), mStatus(0) {
}

Operation::~Operation() {
}

void Operation::setDaqController(DaqController* daq) {
  mDaq = daq;
  mStatus.set(0);
}

int Operation::initialize() {
  if (!mStatus.test(0)) {
    std::cout<<"DaqController& mDaq is not defined."<<std::endl;
    return -1;
  }
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kINFO)) {
    std::cout<<" >> Operation Initialize"<<std::endl;
  }
  mDaq->initialize(); // create Buffer
  mStatus.set(1);
  return 0;
}

int Operation::configure() {
  int ret=0;
  if (!mStatus.test(1)) {
    std::cout<<"Operation::configure() : ";
    std::cout<<"Retry AFTER calling Operation::initialize()"<<std::endl;
    return -1;
  }
  if (!mStatus.test(3)) {
    std::cout<<"Operation::configure() : Retry AFTER calling Operation::readOperationPar(...)"<<std::endl;
    return -1;
  }
  mDaq->getCommMgrTCP().setIPAddressPort(mOpe_p.IP_ADDRESS, mOpe_p.TCP_PORT);
  if (mOpe_p.FIRMWARE_TYPE==2) {
    mDaq->getCommMgrUDP().setIPAddressPort(mOpe_p.IP_ADDRESS, mOpe_p.UDP_PORT);
  }
  if (!mStatus.test(2)) {
    std::cout<<"WARNING : System is not defined."<<std::endl;
  }
  ret=mDaq->configure();
  return ret;
}

/* Reading Parameters from text files */

int Operation::readSystemConfig(const std::string& filename) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kINFO)) {
    std::cout<<" >> Read System Configuration"<<std::endl;
    std::cout<<"  ...reading filename : "<< filename <<std::endl;
  }
  std::ifstream ifs(filename.c_str());
  if (!ifs) { 
    std::cerr<<"Error in readSystemConfig(...) : unable to open input file:"
	     << filename <<std::endl;
    return -1;
  }
  
  std::string line, word1, word2;
  while (getline(ifs, line)) {
    // std::cout<< line <<std::endl;
    std::istringstream stream(line);
    while (stream >> word1 >> word2) {
      // std::cout<< word1 <<", "<< word2 <<std::endl;
      if (word1=="##") {
	// std::cout<<"found ## -> comments -> read next line"<<std::endl;
	break;
      }
      if (word1=="Module") {
	int mid[1], cid[4];
	std::sscanf(word2.c_str(), "%d:", &mid[0]);
	mSystem.createModules(mid, 1);
	int i=0;
	while (stream >> word1) {
	  if (word1=="##") {
	    // std::cout<<"found ## -> comments -> read next line"<<std::endl;
	    break;
	  }
	  std::sscanf(word1.c_str(), "%d", &cid[i]);
	  mSystem.createChips(mid[0], cid, i+1);
	  i++;
	}
      }
    }
  }
  ifs.close();
  if (gconfig && gconfig->output(GlobalConfig::kINFO)) {
    mSystem.print();
  }
  mStatus.set(2);
  return 0;
}

void setParameter(std::string& word1, std::string& word2,
		  int& p, const std::string& name) {
  if (word1==name) {
    std::sscanf(word2.c_str(), "%d", &p);  
    // std::cout<<"p = "<< p <<std::endl;
  }
  return;
}

int Operation::readOperationPar(const std::string& filename) {
  GlobalConfig* gconfig = GlobalConfig::get();
  if (gconfig && gconfig->output(GlobalConfig::kINFO)) {
    std::cout<<" >> Read Operation Parameters"<<std::endl;
    std::cout<<"  ...reading filename : "<< filename <<std::endl;
  }
  std::ifstream ifs(filename.c_str());
  if (!ifs) {
    std::cerr<<"Error in readOperationPar(...) : unable to open input file:"
	     << filename <<std::endl;
    return -1;
  }

  std::string line, word1, word2;
  while (getline(ifs, line)) {
    // std::cout<< line <<std::endl;
    std::istringstream stream(line);
    while (stream >> word1 >> word2) {
      // std::cout<< word1 <<", "<< word2 <<std::endl;
      if (word1=="##") {
	// std::cout<<"found ## -> comments -> read next line..."<<std::endl;
	break;
      }
      if (word1=="IP_Address:")	 mOpe_p.IP_ADDRESS=word2;
      setParameter(word1, word2, mOpe_p.TCP_PORT, "TCP_Port:");
      setParameter(word1, word2, mOpe_p.UDP_PORT, "UDP_Port:");
      setParameter(word1, word2, mOpe_p.FIRMWARE_TYPE, "Firmware_Type:");
    }
  }
  ifs.close();
  if (gconfig && gconfig->output(GlobalConfig::kINFO)) {
    printOperationPar(std::cout);
  }
  mStatus.set(3);
  return 0;
}

void Operation::printOperationPar(std::ostream& out) {
  out <<std::endl;
  out <<" *-------------------------------*"<<std::endl;
  out <<"      Operation Parameters"<<std::endl;
  out <<"  -------------------------------"<<std::endl;
  out <<"   IP Address    : "<< mOpe_p.IP_ADDRESS <<std::endl;
  out <<"   TCP Port      : "<< mOpe_p.TCP_PORT <<std::endl;
  if (mOpe_p.FIRMWARE_TYPE==2) {
    out <<"   UDP Port      : "<< mOpe_p.UDP_PORT <<std::endl;
  }
  out <<"   Firmware Type : "<< mOpe_p.FIRMWARE_TYPE <<std::endl;
  out <<" *-------------------------------*"<<std::endl;
  out <<std::endl;
}


int Operation::finalize() {
  return 0;
}

int Operation::enableDout(int chipID) {
  mDaq->writeRegister(chipID, 20, 0x2641);
  mDaq->writeRegister(chipID,  1, 0x0000);
  mDaq->writeRegister(chipID,  2, 0x2800);
  mDaq->writeRegister(chipID,  3, 0xffff);
  mDaq->writeRegister(chipID,  4, 0xffff);
  mDaq->writeRegister(chipID,  5, 0xd4ff);
  mDaq->writeRegister(chipID,  6, 0x7cd4);
  mDaq->writeRegister(chipID,  7, 0xff58);
  mDaq->writeRegister(chipID,  8, 0xf258);
  mDaq->writeRegister(chipID,  9, 0x00aa);
  mDaq->writeRegister(chipID, 10, 0x4c4c);
  mDaq->writeRegister(chipID, 11, 0x56d4);
  mDaq->writeRegister(chipID, 12, 0x7800);
  // mDaq->writeRegister(chipID, 13, 0x);
  mDaq->writeRegister(chipID, 14, 0xd5b7);
  mDaq->writeRegister(chipID, 15, 0x1a96);
  mDaq->writeRegister(chipID, 16, 0x0038);
  mDaq->writeRegister(chipID, 17, 0x002d);
  mDaq->writeRegister(chipID, 18, 0x00ff);
  mDaq->writeRegister(chipID, 19, 0x0600);
  mDaq->writeRegister(chipID, 20, 0x0041);
  // mDaq->writeRegister(chipID, 21, 0x);
  // mDaq->writeRegister(chipID, 22, 0x);
  mDaq->writeRegister(chipID, 23, 0x0000);
  mDaq->writeRegister(chipID, 24, 0x0000);
  mDaq->writeRegister(chipID, 25, 0x0500);
  mDaq->writeRegister(chipID, 26, 0x0048);
  mDaq->writeRegister(chipID, 27, 0x8000);
  mDaq->writeRegister(chipID, 28, 0x8206);
  mDaq->writeRegister(chipID, 29, 0x0007);
  mDaq->writeRegister(chipID, 30, 0x0000);
  mDaq->writeRegister(chipID, 31, 0xf400);
  mDaq->writeRegister(chipID, 32, 0x0000);
  mDaq->writeRegister(chipID, 33, 0x0000);
  mDaq->writeRegister(chipID, 34, 0x000d);
  // mDaq->writeRegister(chipID, 35, 0x);

  return 0;
}


