/*
  System.cxx
*/
#include <cstdio>
#include <iostream>
#include <sstream>
#include <map>
#include <vector>
#include "PixelDAQ/System.hxx"
#include "PixelDAQ/Module.hxx"

System::System() {
}

System::~System() {
}

void System::createModules(int* moduleID, int n) {
  mModule_m.clear(); // delete
  for (int i=0; i<n; ++i) {
    mModule_m[moduleID[i]] = new Module(moduleID[i], true);
  }
  return;
}

void System::createChips(int moduleID, int* chipID, int n) {
  mModule_m.at(moduleID)->createChips(chipID, n);
  return;
}

void System::print(std::ostream& out) {
  out <<std::endl;
  out <<" *-----------------------------*"<<std::endl;
  out <<"      System construction" <<std::endl;
  out <<"  -----------------------------"<<std::endl;
  out <<"   number of Modules     : "<< numberOfModules() <<std::endl;
  out <<"   number of FE-I4 chips : "<< numberOfChips() <<std::endl;
  out <<" *-----------------------------*" <<std::endl;
  out <<std::endl;
}

int System::numberOfModules() {
  int n = mModule_m.size();
  return n;
}

int System::numberOfChips(int moduleID) {
  int n = mModule_m.at(moduleID)->numberOfChips();
  return n;
}

int System::numberOfChips() {
  int n(0);
  std::map<int, Module*>::iterator p;
  for (p=mModule_m.begin(); p!=mModule_m.end(); ++p) {
    n += (p->second)->numberOfChips();
  }
  return n;
}

void System::setRegister(int moduleID, int chipID, int regAdd,
			 unsigned int data) {
  mModule_m.at(moduleID)->setRegister(chipID, regAdd, data);
  return;
}

unsigned int System::getRegister(int moduleID, int chipID, int regAdd) const {
  unsigned int value = mModule_m.at(moduleID)->getRegister(chipID, regAdd);
  return value;
}




 
