#ifndef __CommunicationMgr_hxx__
#define __CommunicationMgr_hxx__
/*
  CommunicationMgr.hxx
*/
#include <map>
#include <string>
#include "CommBase/CommandSet.hxx"
#include "CommBase/A6Message.hxx"
#include "CommBase/CommBuffer.hxx"

class CommunicationMgr {
public:
  CommunicationMgr();
  CommunicationMgr(const std::string& socket_type, unsigned int buffer_size);
  ~CommunicationMgr();

  void createBuffer(unsigned int n, const std::string& socket_type);

  void setIPAddressPort(const std::string ipaddress, int port);
  void setSocket(int fd);

  int getSocket() const { return mSocket; }
  CommBuffer* buffer()  { return mBuffer; }
  const CommBuffer* buffer() const { return mBuffer; }

  int bind();
  int listen();
  CommunicationMgr* accept();
  int connect();
  int createUDPSocket();
  int close();
 
  int write(CommandSet& cmd);
  int read(CommandSet& cmd);

  int write(A6Message& msg);
  int read(A6Message& msg);

private:
  std::string mIPAddress;
  int mPort;
  int mSocket;
  std::string mSocketType;
  int mMessageNumber;
  CommBuffer* mBuffer;

  std::map<int, CommunicationMgr*> mSubMgrs;
};

#endif // __CommunicationMgr_hxx__
