#ifndef __DaqControllerH_hxx__
#define __DaqControllerH_hxx__
/*
  DaqControllerH.hxx 
  
  last updated : 2017.02.13
*/
#include "PixelDAQ/DaqController.hxx"

class DaqControllerH : public DaqController {
public:
  /* for SiTCP UDP communication */
  enum CommandType   { kREAD=0xc0, kWRITE=0x80, kREPLY=0xc8, kACK=0x88 };
  enum FastCommand { kBCReset=0x03, kECReset=0x05, kCalPulse=0x07 };
  enum SlowCommand { kRdRegister   = 0x09,
		     kWrRegister   = 0x0c,
		     kGlobalReset  = 0x13,
		     kGlobalPulse  = 0x15,
		     kSetRunMode   = 0x18,
		     kSetConfMode  = 0x18,
		     kWrFERegister = 0x80  };


public:
  DaqControllerH();
  ~DaqControllerH();

  int initialize();
  int configure();
  int finalize();

  /* List of commands for FE-I4 chips */;
  int BCReset();
  int ECReset();
  int calPulse() { return 0; } // to be updated...?
  unsigned int readRegister(int chipID, int gr_address);
  int writeRegister(int chipID, int gr_address, unsigned short data);
  int writeFrontEnd(int chipID, char* senddata, char* recvdata,
		    bool output=false);
  int globalReset(int chipID);
  int globalPulse(int chipID, int width);
  int setRunMode(int chipID);
  int setConfMode(int chipID);

public:
  /* List of command for firmware */
  unsigned int readStatus(int add);
  void setChipMask(int chipMask) { mMask = chipMask; }  // for software
  int sendChipMask(int chipMask);
  int sendChipID(int chip_index, int chipID);
  int chkCmdStatus(int address);
  int chkDoutReadiness(int chip);

  /* List of special command */
  int enableDout(int chipID); // initialize chip global register

public:
  int sendUDPCommand(int rw, int cmdid, int plsize, int address, char* payload);
  int writeUDPCommand(int cmdid, int plsize, int address, char* payload);
  unsigned int readUDPCommand(int cmdid, int address);

private:
  int mMask;
};

#endif  // __DaqControllerH_hxx__


